@@ -5,7 +5,7 @@
 
 package XML::Compile::Schema;
 use vars '$VERSION';
-$VERSION = '1.49';
+$VERSION = '1.51';
 
 use base 'XML::Compile';
 
@@ -296,15 +296,24 @@ sub compileType($$@)
     my ($ns, $local)   = unpack_type $type;
 
     my $SchemaNS = SCHEMA2001;
-    $self->importDefinitions( <<_DIRTY_TRICK );
+
+    my $defs     = $ns ? <<_DIRTY_TRICK1 : <<_DIRTY_TRICK2;
 <schema xmlns="$SchemaNS"
    targetNamespace="$ens"
    xmlns:tns="$ns">
   <element name="$elocal" type="tns:$local" />
 </schema>
-_DIRTY_TRICK
+_DIRTY_TRICK1
+<schema xmlns="$SchemaNS"
+   targetNamespace="$ens"
+   elementFormDefault="unqualified"
+   >
+  <element name="$elocal" type="$local" />
+</schema>
+_DIRTY_TRICK2
 
-     $self->compile($action, $elem, %args);
+    $self->importDefinitions($defs);
+    $self->compile($action, $elem, %args);
 }
 
 
