@@ -8,7 +8,7 @@ no warnings 'recursion';
 
 package XML::Compile::Schema::BuiltInTypes;
 use vars '$VERSION';
-$VERSION = '1.50';
+$VERSION = '1.51';
 
 use base 'Exporter';
 
@@ -305,7 +305,15 @@ $builtin_types{sloppy_float} =
 
 $builtin_types{base64Binary} =
  { parse   => sub { eval { decode_base64 $_[0] } }
- , format  => sub { eval { encode_base64 $_[0], '' } }
+ , format  => sub {
+       my $a = $_[0];
+       eval { utf8::downgrade($a) };
+       if($@)
+       {   error __x"use Encode::encode() for base64Binary field at {path}"
+             , path => $_[2];
+       }
+       encode_base64 $a, '';
+    }
  , check   => sub { !$@ }
  , example => 'decoded bytes'
  , extends => 'anyAtomicType'
@@ -339,7 +347,7 @@ my $date = qr/^ $yearFrag \- $monthFrag \- $dayFrag $timezoneFrag? $/x;
 
 $builtin_types{date} =
  { parse   => \&_collapse
- , format  => sub { $_[0] =~ /\D/ ? $_[0] : strftime("%Y-%m-%d", gmtime $_[0])}
+ , format  => sub { $_[0] =~ /^[0-9]+$/ ? strftime("%Y-%m-%d", gmtime $_[0]) : $_[0]}
  , check   => sub { (my $val = $_[0]) =~ s/\s+//g; $val =~ $date }
  , example => '2006-10-06'
  , extends => 'anyAtomicType'
