@@ -4,7 +4,7 @@
 # Pod stripped from pm file by OODoc 2.01.
 package XML::Compile::Translate::Reader;
 use vars '$VERSION';
-$VERSION = '1.49';
+$VERSION = '1.51';
 
 use base 'XML::Compile::Translate';
 
@@ -14,7 +14,7 @@ no warnings 'once', 'recursion';
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
 use List::Util   qw/first/;
-use Scalar::Util qw/weaken/;
+use Scalar::Util qw/weaken blessed/;
 
 use XML::Compile::Util qw/pack_type odd_elements type_of_node SCHEMA2001i/;
 use XML::Compile::Iterator ();
@@ -78,7 +78,7 @@ sub makeElementWrapper
 {   my ($self, $path, $processor) = @_;
     # no copy of $_[0], because it may be a large string
     sub { my $tree;
-          if(ref $_[0] && UNIVERSAL::isa($_[0], 'XML::LibXML::Iterator'))
+          if(blessed $_[0] && $_[0]->isa('XML::LibXML::Iterator'))
           {   $tree = $_[0];
           }
           else
@@ -139,6 +139,7 @@ sub makeSequence($@)
     sub { my $tree = shift;
           my @res;
           my @do = @pairs;
+
           while(@do)
           {   my ($take, $do) = (shift @do, shift @do);
               push @res, ref $do eq 'BLOCK'
@@ -224,8 +225,9 @@ sub makeChoice($@)
               or error __x"choice needs more elements at {path}"
                    , path => $path, _class => 'misfit';
 
+
           my @elems = sort keys %do;
-          trace "choose element from @elems or fix special" if @elems;
+          trace "choose element from @elems or fix special at $path" if @elems;
           trace "failed specials in choice: $_" for @special_errors;
 
           error __x"no applicable choice for `{tag}' at {path}"
@@ -309,7 +311,6 @@ sub makeAll($@)
 sub makeBlockHandler
 {   my ($self, $path, $label, $min, $max, $process, $kind, $multi) = @_;
 
-#warn "BLOCK $label $min $max";
     # flatten the HASH: when a block appears only once, there will
     # not be an additional nesting in the output tree.
     if($max ne 'unbounded' && $max==1)
@@ -588,7 +589,7 @@ sub _not_processed($$)
 
 sub makeComplexElement
 {   my ($self, $path, $tag, $elems, $attrs, $attrs_any,undef,$is_nillable) = @_;
-my @e = @$elems; my @a = @$attrs;
+#my @e = @$elems; my @a = @$attrs;
 
     my @elems = odd_elements @$elems;
     my @attrs = (odd_elements(@$attrs), @$attrs_any);
@@ -931,7 +932,7 @@ sub makeSubstgroup
     sub { my $tree  = shift;
           my $type  = ($tree ? $tree->currentType : undef)
               or error __x"no data for substitution group {type} at {path}"
-                    , type => $base, path => $path;
+                    , type => $base, path => $path, class => 'misfit';
 
           my $do    = $do{$type} or return ();
           my @subst = $do->[1]($tree->descend);
@@ -1091,7 +1092,7 @@ sub makeHook($$$$$$$)
     return $r unless $before || $replace || $after;
 
     return sub { ($_[0]->node->localName => 'SKIPPED') }
-        if $replace && grep {$_ eq 'SKIP'} @$replace;
+        if $replace && grep $_ eq 'SKIP', @$replace;
 
     my @replace = $replace ? map $self->_decodeReplace($path,$_),@$replace : ();
     my @before  = $before  ? map $self->_decodeBefore($path,$_), @$before  : ();
@@ -1113,7 +1114,7 @@ sub makeHook($$$$$$$)
          : $process->();
 
        @h or return ();
-       my $h = @h==1 && !ref $h[0] ? {_ => $h[0]} : $h[1];  # detect simpleType
+       my $h = @h==1 ? $h[0] : $h[1];  # detect simpleType
        foreach my $after (@after)
        {   $h = $after->($xml, $h, $path, $fulltype);
            defined $h or return ();
@@ -1134,7 +1135,8 @@ sub _decodeReplace($$)
 {   my ($self, $path, $call) = @_;
     return $call if ref $call eq 'CODE';
 
-    error __x"labeled replace hook `{call}' undefined for READER", call=>$call;
+      $call eq 'XML_NODE'  ? sub { ($_[3] => $_[0]) }    # don't parse XML
+    : error __x"labeled replace hook `{call}' undefined for READER",call=>$call;
 }
 
 my %after = 
