@@ -5,7 +5,7 @@
  
 package XML::Compile::Translate::Writer;
 use vars '$VERSION';
-$VERSION = '1.50';
+$VERSION = '1.51';
 
 use base 'XML::Compile::Translate';
 
@@ -19,6 +19,9 @@ use Scalar::Util  qw/blessed weaken/;
 use XML::Compile::Util qw/pack_type unpack_type type_of_node SCHEMA2001i
   SCHEMA2001 odd_elements even_elements/;
 
+use XML::LibXML;
+use Encode        qw/encode/;
+
 
 # Each action implementation returns a code reference, which will be
 # used to do the run-time work.  The principle of closures is used to
@@ -612,6 +615,7 @@ sub makeMixedElement
     my @attrs   = @$attrs;
     my @anya    = @$attrs_any;
     my $nilattr = $is_nillable ? $self->nil($path) : undef;
+    (my $locname = $tag) =~ s/.*\://;
 
     my $mixed = $self->{mixed_elements};
     if($mixed eq 'ATTRIBUTES') { ; }
@@ -643,14 +647,38 @@ sub makeMixedElement
           my $content = delete $copy->{_};
           defined $content or return;
 
-          if(UNIVERSAL::isa($content, 'XML::LibXML::Node')) {}
-          elsif($content eq 'NIL')
-          {   $content = $doc->createAttribute($nilattr => 'true')
+          #XXX there are no regression test for these options
+          my $node;
+          if(blessed $content && $content->isa('XML::LibXML::Node'))
+          {   $node    = $doc->importNode($content);
+          }
+          elsif($is_nillable && $content eq 'NIL')
+          {   # nillable element
+              $node    = $doc->createElement($tag);
+              $node->setAttribute($nilattr => 'true');
+          }
+          elsif($content =~ /\<.*?\>|\&\w+\;/)
+          {   # XXX I do not know a way to fill text nodes without getting
+              # entity encoding on them.  Apparently, libxml2 has a
+              # xmlStringTextNoenc, which is not provided in XML::LibXML.
+              # Now, we need the expensive roundtrip, via the parser.
+
+              my $frag = XML::LibXML->new
+                ->parse_balanced_chunk(encode utf8 => $content);
+
+              $node = $frag->firstChild;
+              if($node->localName ne $locname)
+              {   # <nodes>  -->   <$locname>$nodes</$locname>
+                  my $c = $node;
+                  $node = $doc->createElement($tag);
+                  $node->appendChild($frag);
+              }
           }
           else
-          {   $content = $doc->createTextNode($content);
+          {   # Plain text
+              $node    = $doc->createElement($tag);
+              $node->appendText($content);
           }
-          my $node = $doc->importNode($content);
 
           my @childs;
           for(my $i=0; $i<@attrs; $i+=2)
@@ -727,7 +755,7 @@ sub makeBuiltin
     $check
     ? ( defined $format
       ? sub { defined $_[1] or return undef;
-              my $value = $format->($_[1], $trans);
+              my $value = $format->($_[1], $trans, $path);
               return $value if defined $value && $check->($value);
               error __x$err, value => $value, type => $type, path => $path;
             }
@@ -736,7 +764,7 @@ sub makeBuiltin
             }
       )
     : ( defined $format
-      ? sub { defined $_[1] ? $format->($_[1], $trans) : undef }
+      ? sub { defined $_[1] ? $format->($_[1], $trans, $path) : undef }
       : sub { $_[1] }
       );
 }
